% File:    Data.m
% Autori: Destro,Panara,Pezzotta
% Data:    13/10/99
% Note:   Controllo di un processo

clear all;
close all;

% Valore dei parametri del sistema in esame:

	H 	= 60.96;	% altezza contenitore 	  [cm]
	pamb 	= 1014;		% presssione ambiente 	  [kg/(cm*s^2)]
	R	= 2874.5;	% costante dei gas        [cm^2/(s^2*K)]
	T	= 298;		% temperatura aria	  [K]
	rhol	= 1.5E-3;	% densita' acqua	  [kg/cm^3]
	Aau	= .035;		% sezione aria uscente	  [cm^2]
	Al	= 180.64;	% sezione contenitore	  [cm^2]
	Alu	= 4.5;		% sezione uscita	  [cm^2]
	g	= 980;		% accelerazione gravita'  [cm/s^2]
	k	= 1.4;		% esponente adimensionale [-]

% Valore delle variabili di stato all'equilibrio desiderato:

	hl	= 20;
	paria	= 1111;
	x(1)	= hl;
	x(2)	= paria;

% Valore degli ingressi all'equilibrio (in realta' u(3) non e' controllabile e quindi
% e' un disturbo comunque misurabile):

u(3)=pamb;
u(1)=rhol*Alu*sqrt(2*(((x(2)-u(3))/rhol)+g*x(1)));
u(2)=Aau*x(2)*sqrt(28.8/(R*T))*sqrt((2*k/(k-1))*((u(3)/x(2))^(2/k)-(u(3)/x(2))^((k+1)/k)));

% I valori numerici all'equilibrio sono:

	% u(1)= portante di liquido in ingresso = qli = 2.7711
	% u(2)= portante di aria in ingresso = qai = 0.0897
	% u(3) = pamb = 1014
	
% LINEARIZZAZIONE DEL SISTEMA NON LINEARE:
% Creiamo sys con il file-modello di simulink lineare.mdl
% Successivamente usiamo linmod per ottenere il modello in forma state space 
% linearizzato.
% Attraverso la funzione linmod di matlab verifichiamo i valori delle matrici 
% A, B, C, D del sistema linearizzato:

%					dx/dt = A x + B u
%					y(t) = C x + D u


% Linearizzazione:

% Matrice A:
	
% dhl/dt = -(dhl*rhol*Alu*g)/(rhol*Al*sqrt[2*((paria-pamb)/rhol)+g*hl)]) + 
% 	   - (dparia*Alu)/(rhol*Al*sqrt[2*((paria-pamb)/rhol)+g*hl)]) -->
% 		   dhl/dt = -0.05946 * dhl -0.04045 * dparia

% dparia/dt = -(dhl*paria*rhol*Alu*g)/(H-hl)*(rhol*Al*sqrt[2*((paria-pamb)/rhol)+g*hl)]) +
% 	   _ (1/(H-hl))*{(paria/(rhol*Al))*(-Alu/sqrt[2*((paria-pamb)/rhol)+g*hl)])+(RT/(28.8*Al))*
%	   [-Aau*paria*sqrt[28.8/RT]*1/[2*sqrt[((2*k)/(k-1))*[(pamb/paria)^(2/k)-(pamb/paria)^((k+1)/k)]]]}

%						=====>
	
%		   A=
%			-0.05946	-0.04045
%			-1.61279	-1.09...

% Matrice B:
% dhl/dt = dqli/(rhol*Al) + 0 * dqai + Alu/{(rhol*Al)*sqrt[2*((paria-pamb)/rhol)+g*hl)]} * dpamb -->
%                  dhl/dt = 3.6906 * dqli + 0 * dqai + 0.040454 * dpamb
% dparia/dt = (paria*dqli)/[rhol*Al*(H-hl)] + (RT*dqai)/[28.8*Al*(H-hl)] + ..........*dpamb -->
%		   dparia/dt = 100.0985216 * dqli + 4.019871928 * dqai + ...... * dpamb

%		   				=====>

%		   B=
%			3.6906		0		0.040454
%			100.0985216	4.019871928	......
% Matrice C:
	
% dy1/dt = dhl/dt = dhl + 0 * dparia
 	
% dy2/dt = dparia/dt = 0 * dhl + dparia
 	
%		=====>
 	
%		   C=
%			1	0
%			0	1
 	
% Matrice D:
 	
% Le uscite non sono funzione degli ingressi dqli, dqai, dpamb, quindi
 	
%		   	      =====>
 	
%		   D=
 	
%			0	0	0
%			0	0	0



	[A B C D]=linmod('lineare',x,u);

% Risultati ottenuti con la funzione linmod:

	% A=
	%	-0.0595		-0.0405
	%	-1.6130		-1.0991

	% B=
	%	3.6906		0		0.0405
	%	100.1034	4.0199		1.0989

	% C=
	%	1.0000		0
	%	0		1.0000

	% D=
	%	0		0		0
	%	0		0		0
	
% Possiamo affermare che i valori restituiti dal modello di simulink e quelli da 
% noi calcolati coincidono.

% Ricaviamo ora le FUNZIONI DI TRASFERIMENTO FRA INGRESSI E USCITE


	[num1,den1]=ss2tf(A,B(:,1),C(1,:),D(1,1),1);	    % G11
	[num2,den2]=ss2tf(A,B(:,2),C(1,:),D(1,1),1);    % G12
	[num3,den3]=ss2tf(A,B(:,3),C(1,:),D(1,1),1);		% H31
	[num4,den4]=ss2tf(A,B(:,1),C(2,:),D(1,1),1);	    % G21
	[num5,den5]=ss2tf(A,B(:,2),C(2,:),D(1,1),1);    % G22
	[num6,den6]=ss2tf(A,B(:,3),C(2,:),D(1,1),1);		% H32

% Visualizzo le funzioni di trasferimento G11 - G22 - G12 - G21 e il relativo 
% guadagno statico (s=0)

input('Funzioni di trasferimento G11-G22-G12-G21 e relativo guadagno statico(s=0):(PREMI UN TASTO PER CONTINUARE)');

	sysG11=tf(num1,den1)
	dcgain(sysG11)
	sysG22=tf(num5,den5)
	dcgain(sysG22)
	sysG12=tf(num2,den2)
	dcgain(sysG12)
	sysG21=tf(num4,den4)
	dcgain(sysG21)

	% 	         3.691 s + 0.006813
	% G11(s) =  ---------------------------		G11(0)=62.0694 
	%	     s^2 + 1.159 s + 0.0001098

	% 	          4.02 s + 0.2391
	% G22(s) =  ---------------------------		G22(0)=2.1779e+03
	%	     s^2 + 1.159 s + 0.0001098

	% 	         -0.1626
	% G12(s) =  ---------------------------		G12(0)=-1.4816e+03
	%	     s^2 + 1.159 s + 0.0001098

	% 	      100.1 s - 1.482 * 10^-6
	% G21(s) =  ---------------------------		G21(0)=-0.0135
	%	     s^2 + 1.159 s + 0.0001098

% Visualizzo le funzioni di trasferimento per il disturbo pamb: H31 H32

	input('Funzioni di trasferimento H31 H32: (PREMI UN TASTO PER CONTINUARE)');

	sysH31=tf(num3,den3)
	sysH32=tf(num6,den6)

	% 	     0.04045 s + 7.243 * 10^-6
	% H31(s) =  ---------------------------	
	%	     s^2 + 1.159 s + 0.0001098

	% 	      1.099 s + 9.912 * 10^-5
	% H32(s) =  ---------------------------	
	%	     s^2 + 1.159 s + 0.0001098


% CONFRONTO FRA IL MODELLO NON LINEARE E QUELLO LINEARIZZATO

% Verifichiamo che il sistema non lineare e quello linearizzato offrono all'incirca 
% le stesse risposte allo scalino --> schema e grafici nel file-modello di simulink: 
% cfr_scalino.mdl .Effettuiamo la simulazione con i valori degli scalini di ingresso 
% rispettivamente pari a:
% (0.0277, 0.27711, 2.7711) per u(1)=qli; 
% (8.9703*10^-4, 0.009,0.0897) per u(2)=qai;
% (10.14, 101.4, 1014) per u(3)=pamb.
% Verifichiamo cosi' che per piccoli valori dello scalino le risposte coincidono,
% mentre aumentandoli le risposte si discostano sempre piu':
% Differenza altezza liquido a regime=(3.9*10^-3 su h~0.44, 0.375 su h~4-4.5,
% 21.8 su h~22.5-44.5);
% Differenza pressione aria a regime=(0.018 su p~12, 1.813 su p~120-122, 
% 159 su p~1200-1360 );
% Per operare questi confronti variare il valore qui di seguito di par (0.01, 0.1, 1)
%, salvare nuovamente il file Data.m, rieseguirlo da matlab e rilanciare la 
% simulazione da cfr_scalino.mdl oppure fornire il valore di par direttamente 
% da matlab e rilanciare la simulazione da cfr_scalino.mdl.

	par=0.01;

% Riduco la matrice B in modo da renderla quadrata (indipendente dal disturbo p-amb):

	B1=B(:,1:2);

% Calcolo la matrice G delle funzioni di trasferimento:

	G=C*inv(-A)*B1;

% Calcolo la matrice dei guadagni relativi RGA per vedere come realizzare uno schema 
% di controllo decentralizzato (associare ad ogni uscita un solo ingresso):

	input('MATRICE DEI GUADAGNI RELATIVI : (PREMI UN TASTO PER CONTINUARE)');

	RGA=G.*(inv(G))'

	% Poiche' si vede che la matrice RGA e' approssimata dalla matrice identita'
	% I, il sistema puo' essere modificato in modo che y1/u2=0 ed y2/u1=0.

% Per disaccoppiare il sistema, occorre progettare uno schema di disaccoppiamento:
% utilizziamo uno schema di disaccoppiamento all'indietro 
% (schema e grafici nei file-modello regolato.mdl regolatoB.mdl regolato2.mdl 
% e regolato2B.mdl)

input('Funzioni di trasferimento dei disaccoppiatori D12 e D21 : (PREMI UN TASTO PER CONTINUARE)');

sysD12=-sysG12/sysG11

numD12=-num2;
denD12=num1;

sysD21=-sysG21/sysG22

numD21=-num4;
denD21=num5;

input('Funzioni di trasferimento dei disaccoppiatori D12 e D21 semplificate: (PREMI UN TASTO PER CONTINUARE)');

sysD12=tf(numD12, denD12)
sysD21=tf(numD21, denD21)

% 	     0.1626 s^2 + 0.1884 s + 1.785 * 10^-5                        0.1626
% D12(s) =  -------------------------------------------------  =   --------------------	
%	     3.691 s^3 + 4.283 s^2 + 0.008299 s + 7.4 *10^-7        3.691 s + 0.006813

% 	     -100.1 s^3 - 116 s^2 - 0.01099 s + 1.627 * 10^-10        -100.1 s + 1.482 * 10^-6
% D21(s) =  ---------------------------------------------------  =   ---------------------------	
%	      4.02 s^3 + 4.896 s^2 + 0.2774 s + 2.624 * 10^-5      	  4.02 s + 0.2391

% Per rendere il sistema il piu' possibile immune dalle azioni del disturbo, 
% si progettano due compensatori del disturbo M31 ed M32 
% (schema e grafici nei file-modello regolatoB.mdl e regolato2B.mdl)

input('Funzioni di trasferimento dei compensatori M31 e M32 : (PREMI UN TASTO PER CONTINUARE)');

sysM31=tf(-num3,num1)
sysM32=tf(-num6,num5)

% 	     -0.04045 s - 7.243 * 10^-6
% M31(s) =  ----------------------------	
%	          3.691 s + 0.006813

% 	      -1.099 s - 9.912 * 10^-5
% M32(s) =  ---------------------------	
%	           4.02 s + 0.2391

% Con l'uso dei compensatori per il sistema non lineare si ha una risposta migliore
% al disturbo; anche per il sistema linearizzato le risposte alla variazione 
% del disturbo migliorano.
% Le risposte coincidono nel caso di variazione nulla del disturbo sia con 
% il sistema non lineare sia con quello linearizzato. 

% Successivamente si progetteranno rispettivamente i regolatori R1 solo in funzione
% di G11 ed R2 solo in % funzione di G22 per avere uno schema di 
% controllo decentralizzato

% Ricavo i poli, gli zeri ed il rho delle funzioni di trasferimento G11 e G22:

[A11 B11 C11 D11]=tf2ss(num1,den1);	% Conversioni da modello trnsfer-function a 
[A22 B22 C22 D22]=tf2ss(num5,den5);	% modello state-space
[A12 B12 C12 D12]=tf2ss(num2,den2);
[A21 B21 C21 D21]=tf2ss(num4,den4);

[z1 p1 k1]=tf2zp(num1,den1);	% Conversioni da modello state-space a modello
[z2 p2 k2]=tf2zp(num5,den5);	% zero-pole-gain 

% Dopo aver visto i valori dei poli, degli zeri e del rho di G11 e G22 
% progettiamo i regolatori R1 ed R2. 
% La soluzione migliore appare quella di progettare dei PI che coprano con 
% lo zero il polo piu' in alta frequenza perche' cio' comporta uno spostamento
% dei poli in anello chiuso piu' verso l'alta frequenza  
% (come evidenziato dal luogo delle radici).
% Il problema si ridurra' in entrambi i casi a determinare un guadagno (rho) 
% che permetta di avere stabilita',e quindi di avere il polo piu' in bassa frequenza
% in anello chiuso a sinistra e vicino allo zero,e permetta di ottenere 
% una risposta allo scalino abbastanza veloce. 
 
% Per fare cio' facciamo rlocus ed rlocfind sulle funzioni L1 ed L2 con kR1=kR2=1!!
% (che rappresentano rispettivamente rho1 e rho2). 
% Valutiamo un accettabile valore per questi due parametri.

scelta=input('Vuoi analizzare L1 ed L2 per scegliere il rho opportuno? (y=1 - n=0) ');
if (scelta==1),
	numL1=[1 0.0018]*k1*0.8632;	%0.8632=-1/zR1
	denL1=[1 0.0001 0];

	input('Funzioni di trasferimento di L1=R1*G11 con rho=1: (PREMI UN TASTO PER CONTINUARE)');
	sysL1=tf(numL1,denL1)
	cont=1;
	while (cont==1),
		figure(1);
		title('Luogo delle radici di L1 (rho=1) - (zoom on)');
		rlocus(sysL1)
		zoom on;grid on;
		input('(PREMI UN TASTO PER CONTINUARE)');
		[rhonew,poli]=rlocfind(sysL1)
		cont=input('continui?(y=1/n=0) : ');
	end	% Fine di:while (cont==1)
		
	numL2=[1 0.059]*k2*0.8632;	%0.8632=-1/zR2
	denL2=[1 0.0001 0];

	input('Funzioni di trasferimento di L2=R2*G22 con rho=1: (PREMI UN TASTO PER CONTINUARE)');
	sysL2=tf(numL2,denL2)
	cont=1;
	while (cont==1),
		figure(2);
		title('Luogo delle radici di L2 (rho=1) - (zoom on)');
		rlocus(sysL2)
		zoom on;grid on;
		input('(PREMI UN TASTO PER CONTINUARE) ');
		[rhonew,poli]=rlocfind(sysL2)
		cont=input('continui?(y=1/n=0) : ');
	end	% Fine di:while (cont==1)
	close all;
	kR1=input('Introduci il valore di rho per R1 ed L1 (consigliato->0.01 oppure 0.07): ');
	kR2=input('Introduci il valore di rho per R2 ed L2 (consigliato->0.01 oppure 0.07): ');
end	% Fine di:if (scelta==1)
cont2=1;		% Ciclo per permettere di cambiare i val. dei rho!!
while (cont2==1),	% N.B. E' intersecato col ciclo per studiare il luogo delle radici.
if (scelta==0),
	close all;
	kR1=input('Introduci il valore di rho per R1 ed L1 (consigliato->0.01 oppure 0.07): ');
	kR2=input('Introduci il valore di rho per R2 ed L2 (consigliato->0.01 oppure 0.07): ');
end	% Fine di:if (scelta==0)

% OSSERVAZIONI SUL LUOGO DELLE RADICI
% Con rlocfind abbiamo verificato che per ottenere la situazione sopra descritta
% di posizionamento delle singolarita' occorre scegliere rho1>0.0022 e rho2>0.068.	
% Alla fine abbiamo scelto kR1=0.01 e kR2=0.01 come valore dei rho per 
% i due regolatori (Anche se per la press. dell'aria si hanno poli compl. coniug.)
% perche' volevamo ridurre il piu' possibile le variazioni 
% istantanee delle var di controllo: valori piu' bassi davano problemi di 
% esecuzione delle simulazioni.
% Tuttavia e' possibile verificare la risposta del sistema 
% anche con altri valori dei rho.Se si aumenta il valore di rho1 si ottiene una 
% risposta piu' veloce del livello del liquido, ma si ha anche un piu' elevato 
% valore negativo della qa e con variazione istantanea!! 
% Se si aumenta rho2, si eliminano le oscillazioni, si velocizza la risposta,
% ma si hanno delle variazioni istantanee di qa con valori elevati!!
% Altra coppia di valori consigliati e' kR1=0.07 e kR2=0.07, in modo da non 
% avere per la press. aria dei poli compl. coniug., accelerare le risposte ed 
% eliminare le oscillazioni sulla pressione dell'aria.


	zR1=-1.1585;
	pR1=0;

	numR1=[-1/zR1 1]*kR1;
	denR1=[1 0];

	input('Funzioni di trasferimento del regolatore R1 e di L1=R1*G11: (PREMI UN TASTO PER CONTINUARE)');

	sysR1=tf(numR1,denR1)

	numL1=[1 0.0018]*kR1*k1*0.8632;
	denL1=[1 0.0001 0];

	sysL1=tf(numL1,denL1)

	% A questo punto valutiamo la risposta allo scalino, la posizione dei poli e 
	% degli zeri ed il diagramma di Bode dell'anello chiuso
	% AC1=L1*G11 (retroazionato negativamente)

	figure(1)
	AC1=feedback(sysL1,1);
	[y,t]=step(AC1,(1:2000));
	plot(t,y)
	title('Risposta allo scalino dell''anello chiuso sul livello del liquido');
	zoom on;grid on;
	hold on;
	figure(2)
	rlocus(AC1);
	title('Posizione delle singolarita'' dell''anello chiuso sul livello del liquido');
	zoom on;sgrid;grid on;
	figure(3)
	w=logspace(-2,2,1500);
	zoom on;grid on;
	zero=tf(1,1);
	bode(AC1,'-m',sysL1,'-g',zero,'-b',w);
	title('Diagramma di Bode di L1(s) ed F1(s)');

	
	zR2=-1.1585;
	pR1=0;

	numR2=[-1/zR2 1]*kR2;
	denR2=[1 0];

	input('Funzioni di trasferimento del regolatore R2 e di L2=R2*G22: (PREMI UN TASTO PER CONTINUARE)');

	close all;
	sysR2=tf(numR2,denR2)

	numL2=[1 0.059]*kR2*k2*0.8632;
	denL2=[1 0.0001 0];

	sysL2=tf(numL2,denL2)

	% A questo punto valutiamo la risposta allo scalino, la posizione dei poli e 
	% degli zeri ed il diagramma di Bode dell'anello chiuso
	% AC2=L2*G22 (retroazionato negativamente)

	figure(1)
	AC2=feedback(sysL2,1);
	[y,t]=step(AC2,(1:2000));
	plot(t,y)
	title('Risposta allo scalino dell''anello chiuso sulla pressione dell''aria');
	zoom on;grid on;
	hold on;
	figure(2)
	rlocus(AC2);
	title('Posizione delle singolarita'' dell''anello chiuso sulla pressione dell''aria');
	zoom on;sgrid;grid on;
	figure(3)
	w=logspace(-2,2,1500);
	zoom on;grid on;
	zero=tf(1,1);
	bode(AC2,'-m',sysL2,'-g',zero,'-b',w);
	title('Diagramma di Bode di L2(s) ed F2(s)');

	cont2=input('Vuoi variare ancora rho1 e rho2 e vederne le corrispondenti risp. allo scalino, posiz. delle singolarita'' e diagr. di Bode?(y=1/n=0) : ');
	close all;
	scelta=0;
end	% Fine di:while (cont2==1) - ciclo per poter cambiare i val. dei rho.

% OSSERVAZIONI SUI MODELLI E SUI CORRISPONDENTI GRAFICI:
	
% POICHE' LA SIMULAZIONE PARTE DA UNO STATO INDETERMINATO, ATTENDIAMO CHE IL 
% SISTEMA SI ASSESTI DA SOLO PRIMA DI APPLICARE UNA VARIAZIONE A SCALINO
% DEGLI INGRESSI.
% TALI VARIAZIONI SONO:
%	
%	a) per t=1000 variazione del disturbo del 10% del valore nominale  (1014)
%	b) per t=2000 variazione dell' hl0 del 10% del valore di equilibrio (20)
%	c) per t=4000 variazione della paria0 del 10% del valore di equilibrio (1111)

% Si sono appositamente distanziate di molto le variazioni degli ingressi per 
% valutare se il sistema risulta effettivamente disaccoppiato e per meglio 
% apprezzare i tempi di assestamento.

% File-modello regolato.mdl : (SISTEMA NON LINEARE - Non contiene i compensatori 
% M31 ed M32 per il disturbo)
% Con i valori kR1=0.01 e kR2=0.01 si ha una soddisfacente risposta del sistema: 
% dopo un transitorio di circa 2000 [sec] sia per hl che per paria le due uscite 
% si portano al nuovo valore a seguito di una variazione del corrispondente 
% ingresso a scalino, confermando quanto visto dalla risposta a scalino 
% visualizzata in fase di progetto dei regolatori.In realta' la press. si assesta in 
% circa 300 sec.
% Entrambe le uscite risentono di una variazione del disturbo pamb, 
% subendo una variazione anche le qli e qa.
% Le variazioni delle singole uscite a seguito di una variazione a scalino 
% dell'altro ingresso non possono considerarsi trascurabili.
% Inoltre si nota una brusca variazione dei valori di qa all'istante t=2000 
% (quando si ha la variazione a scalino del livello del liquido) e al tempo t=4000.
	

% File-modello regolatoB.mdl : (SISTEMA NON LINEARE - Contiene i compensatori 
% M31 ed M32 per il disturbo)
% Con i valori kR1=0.01 e kR2=0.01 si ha una soddisfacente risposta del sistema:
% dopo un transitorio di circa 2000 [sec] le due uscite si portano al nuovo 
% valore a seguito di una variazione del corrispondente ingresso a scalino.
% Entrambe le uscite risentono di una variazione del disturbo pamb,
% riassestandosi dopo un transitorio di tempo.
% Le variazioni delle singole uscite a seguito di una variazione a scalino 
% dell'altro ingresso possono considerarsi trascurabili.
% Tuttavia si nota una brusca variazione dei valori di qli e qa all'istante t=2000 
% (quando si ha la variazione a scalino del livello del liquido) e al tempo t=4000.5)
	
% File-modello regolato2.mdl : (SISTEMA LINEARIZZATO - Non contiene i compensatori 
% M31 ed M32 per il disturbo)
% Con i valori kR1=0.01 e kR2=0.01 si ha una soddisfacente risposta del sistema: 
% dopo un transitorio di circa 2000 [sec] sia per hl che per paria le due uscite 
% si portano al nuovo valore a seguito di una variazione del corrispondente ingresso
% a scalino, confermando quanto visto dalla risposta a scalino visualizzata 
% in fase di progetto dei regolatori.In realta' la press. si assesta in 
% circa 300 sec.
% Entrambe le uscite risentono di una variazione del disturbo pamb, 
% subendo una variazione anche le qli e qa.
% Le variazioni delle singole uscite a seguito di una variazione a scalino 
% dell'altro ingresso possono considerarsi trascurabili.
% Tuttavia si nota una brusca variazione dei valori di qa all'istante t=2000 
% (quando si ha la variazione a scalino del livello del liquido) e al tempo t=4000.

% File-modello regolato2B.mdl : (SISTEMA LINEARIZZATO - Contiene i compensatori 
% M31 ed M32 per il disturbo)
% Con i valori kR1=0.01 e kR2=0.01 si ha una soddisfacente risposta del sistema: 
% dopo un transitorio di  circa 2000 [sec] sia per hl che per paria le due uscite 
% si portano al nuovo valore a seguito di una variazione del corrispondente
% ingresso a scalino, confermando quanto visto dalla risposta a scalino 
% visualizzata in fase di progetto dei regolatori.In realta' la press. si assesta in 
% circa 300 sec.
% Adesso le uscite non risentono piu' delle variazioni del disturbo pamb 
% (ad eccezione di una piccola variazione di qli).
% Le variazioni delle singole uscite a seguito di una variazione a scalino 
% dell'altro ingresso possono considerarsi trascurabili. 

% PER EVITARE DI AVERE LE VARIAZIONI ISTANTANEE A SCALINO DELLE VAR DI CONTROLLO SI 
% POTREBBERO FILTRARE I GRADINI DI INGRESSO!!(esempio in regolato_filtrato.mdl dove si
% puo' inserire pure un saturatore su qa per evitarne valori negativi).
% Si e' inserita la variaz. del disturbo a t=5000 anziche' per t=1000.


	% SAREBBE OPPORTUNO EVITARE CHE LE VAR DI CONTROLLO ASSUMANO VALORI NEGATIVI:
	% PER FAR CIO' SI INTRODUCONO DEI SATURATORI, MA
	% IN QUESTO MODO NON SI RIESCE PIU' A DISACCOPPIARE COMPLETAMENTE IL SISTEMA.
	% I saturatori sono gia' presenti nei modelli regolatoB.mdl e regolato2B.mdl.
	% Per avere i risultati precedentemente commentati 
	% occorre porre il loro limite inferiore a un valore negativo (es. -1000000)
	% anziche' a 0.!!E' stato inoltre corretto il 
	% denominatore del disaccoppiatore D21 per evitare problemi alla simulazione!!
	% (Per avere i dati prec commentati occorre 
	% ricorreggere tale denominatore e staccare il saturatore dallo schema a 
	% blocchi).
	% Con i valori di kR1=kR2=0.01 si vede che:
	% 	- con il sis. linearizzato il sistema si stabilizza seppur con tempi 
	% 	  piu' lunghi.
	% 	- con il sis. non lineare il sistema si stabilizza seppur con tempi 
	%	  piu' lunghi.
	
	% IN ALTERNATIVA E' POSSIBILE APPLICARE UN PASSAGGIO MANUALE AUTOMATICO PER 
	% EVITARE DI ATTENDERE CHE IL SISTEMA 
	% SI ASSESTI DA SOLO ALL'AVVIO DELLA SIMULAZIONE, IN MODO DA ACCELERARE IL 
	% TRANSITORIO INIZIALE.
	% PER FARE CIO' OCCORRE PRIMA EFFETTUARE UNO SCHEMA ANTI WIND-UP:
	
		% Utilizziamo uno schema anti-wind up sul sistema precedentemente 
		% realizzato 
		% (schema e grafici nei file-modello regolato_no_w_up.mdl e 
		% regolato2_no_w_up.mdl)

		% ~Da sistemare!!!!!!!!
		% Infatti il sis. linearizzato con saturatori aventi lim. inf.=0  
		% e variaz. del disturbo attivo non funziona!!!

		% File-modello regolato_no_w_up.mdl : (SISTEMA NON LINEARE - 
		% Nessun disaccoppiamento per evitare problemi)
		% Con i valori kR1=kR2=0.01 si innascano delle oscillazioni permanenti.
		% Proviamo allora a variare i valori di kR1 e 
		% di kR2:ponendoli entrambi a 0.2 si hanno delle oscillazioni smorzate.
		% Inoltre il sistema non e' disaccoppiato (anche 
		% perche' privo di disaccoppiatori) e risente dell'eventuale variazione
		% del disturbo.Se tentiamo di compensare 
		% quest'ultimo, otteniamo una risposta peggiore: si hanno oscillazioni 
		% permanenti.Per questo motivo abbiamo eseguito la
		% simulazione senza il disturbo oppure abbiamo cambiato nuovamente i 
		% valori di kR1 e kR2 a 0.5; cosi' facendo si 
		% eliminano le oscillazioni permanenti, ma il disturbo fa in modo che 
		% il sis. non risponda come voluto!!!
		% SI E' USATO UN VALORE NEGATIVO COME LIMITE INFERIORE DEI SATURATORI 
		% DI qa (-200)!!Mettendo tale valore a zero si ha
		% una risposta ancora accettabile e simile a quella precedente, pur di
		% trascurare il disturbo.

		% File-modello regolato2_no_w_up.mdl : (SISTEMA LINEARIZZATO)
		% Con i valori di kR1=kR2=0.01 si ha una soddisfacente risposta alle 
		% variazioni allo scalino e un buon disaccoppiamento,
		% pur avendo brusche variazioni delle var di controllo (come prima!).
		% SI E' USATO UN VALORE NEGATIVO COME LIMITE INFERIORE DEI SATURATORI 
		% DI qa (-2)!!Mettendo tale valore a zero si ha una 
		% simulazione lunghissima fino a giungere ad un errore.
		% Togliendo pero' la variazione del disturbo la simulazione ritorna
		% ad essere buona!!.
	
	% TORNIAMO ALLA REALIZZAZIONE DEL PASSAGGIO MANUALE-AUTOMATICO:
	% (schema e grafici nei file-modello regolato_nw_man_aut.mdl->NON VA!! 
	%  e regolato2_nw_man_aut.mdl->~OK!!)
		
		% File-modello regolato_nw_man_aut.mdl : (SISTEMA LINEARIZZATO)
		% Con i valori di kR1=kR2=0.01 si hanno problemi!!!!!!!!!!!!!!!!!
		
		% File-modello regolato2_nw_man_aut.mdl : (SISTEMA LINEARIZZATO)
		% Con i valori di kR1=kR2=0.01 si ha una soddisfacente risposta alle 
		% variazioni allo scalino e un buon disaccoppiamento,
		% pur avendo brusche variazioni delle var di controllo (come prima!).
		% SI E' USATO UN VALORE NEGATIVO COME LIMITE INFERIORE DEI SATURATORI 
		% DI qa (-10)!!Mettendo tale valore a zero la 
		% risposta del sistema e' ancora buona per il livello del liquido,
		% ma non piu' valida per la pressione dell'aria.


% DISCRETIZZAZIONE DEI REGOLATORI OTTENUTI (senza effettuare lo schema anti wind-up
% e il passaggio man-aut) 

% Dopo aver osservato i diagrammi di Bode dei sistemi in anello chiuso 
% (o eventualmente di L1 ed L2), abbiamo visto che
% l'anello di controllo per la press. dell'aria e' piu' veloce di quello per
% il livello del liquido. Inoltre entrambi hanno attenuazione fra i 20 e i 40 dB per wn
% molto simili.Scegliamo quindi il valore della wn pari a:pi

	% Parametri di campionamento
	
		wn=input('Inserisci il valore di wn (consigliato wn=pi): ');	
				% wn=Pulsazione di Nyquist
		Tc=pi/wn;	% Tempo di campionamento

	% Con wn=pi il valore di Tc e'1
		
	% Digitalizzazione di R1
		
		[numR1d denR1d]=c2dm(numR1,denR1,Tc,'tustin');
		input('Funzioni di trasferimento del regolatore discretizato R1d: (PREMI UN TASTO PER CONTINUARE)');
	close all;
		R1d=tf(numR1d,denR1d,Tc)
		
	% Digitalizzazione di R2
		
		[numR2d denR2d]=c2dm(numR2,denR2,Tc,'tustin');
		input('Funzioni di trasferimento del regolatore discretizato R2d: (PREMI UN TASTO PER CONTINUARE)');
		R2d=tf(numR2d,denR2d,Tc)

	% Digitalizzazione del disaccoppiatore D12
		[numD12d denD12d]=c2dm(numD12,denD12,Tc,'tustin');
		input('Funzioni di trasferimento del disaccoppiatore discretizato D12d: (PREMI UN TASTO PER CONTINUARE)');
		D12d=tf(numD12d,denD12d,Tc)

	% Digitalizzazione del disaccoppiatore D21
		[numD21d denD21d]=c2dm(numD21,denD21,Tc,'tustin');
		input('Funzioni di trasferimento del disaccoppiatore discretizato D21d: (PREMI UN TASTO PER CONTINUARE)');
		D21d=tf(numD21d,denD21d,Tc)
	
	% Digitalizzazione del compensatore del disturbo M31
		[numM31d denM31d]=c2dm(-num3,num1,Tc,'tustin');
		input('Funzioni di trasferimento del compensatore del disturbo discretizato M31d: (PREMI UN TASTO PER CONTINUARE)');
		M31d=tf(numM31d,denM31d,Tc)

	% Digitalizzazione del compensatore del disturbo M32
		[numM32d denM32d]=c2dm(-num6,num5,Tc,'tustin');
		input('Funzioni di trasferimento del compensatore del disturbo discretizato M32d: (PREMI UN TASTO PER CONTINUARE)');
		M32d=tf(numM32d,denM32d,Tc)

	% Filtro per lo scalino d'ingresso
	
	numf=[0 1];
	denf=[1 1];

	% Il sistema discretizzato e' contenuto in discreto.mdl.
	% DA SISTEMARE CON IL DISTURBO PAMB!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	% File-modello discreto.mdl : (SISTEMA DISCRETIZZATO - Contiene i compensatori
	% M31 ed M32 per il disturbo)
	% Con i valori kR1=0.01, kR2=0.01 e Tc=1 si ha una soddisfacente risposta del
	% sistema: dopo un transitorio di 
	% circa 2000 [sec] sia per hl che per paria le due uscite si portano al nuovo
	% valore a seguito di una variazione 
	% del corrispondente ingresso a scalino.
	% Entrambe le uscite risentono di una variazione del disturbo pamb al punto che
	% la simulazione ha problemi.
	% Per questa ragione il disturbo e' posto a zero!!!.
	% Le variazioni delle singole uscite a seguito di una variazione a scalino 
	% dell'altro ingresso non possono considerarsi
	% trascurabili come nel caso continuo. Inoltre si nota una variazione dei 
	% valori di qa all'istante t=2000 (quando
	% si ha la variazione a scalino del livello del liquido) e al tempo t=4000.
	% Per evitare di avere valori negativi delle var di controllo si introduce un 
	% saturatore: la risposta del sistema risulta ancora
	% accettabile.
	 
% CONTROLLO LQ
% (schema e grafici nei file-modello lq_model_lin.mdl lq_model_nonlin.mdl cfr_lq.mdl)

	input('SISTEMA DISCRETIZZATO - Matrici associate (Ad,Bd,Cd,Dd): (PREMI UN TASTO PER CONTINUARE)');
		
	
	[Ad,Bd,Cd,Dd]=c2dm(A,B,C,D,Tc,'zoh')
	
	% Riduco le matrice Bd e Dd in modo da renderle quadrate (indipendente dal disturbo p-amb):

	Bd1=Bd(:,1:2);
	Dd1=Dd(:,1:2);
	
	input('CONTROLLO LQ - Matrici associate (As,Bs,Cs,Ds) - Matrici Qs,Rs utilizzate - matrice Ks di guadagno - Soluz. dell''eq. di riccati Ps - autovalori dell''anello chiuso: (PREMI UN TASTO PER CONTINUARE)');
		
	As=[Ad zeros(2,2); -Cd eye(2,2)]
	Bs=[Bd1; zeros(2,2)]
	Cs=Cd
	Ds=Dd1
	Qs=[10 25 34 1;1000 800 21 5; 3 1 21 3; 1 1 1 1]
	Rs=[100 500;250 10000]

	
	[Ks,Ps,Es]=dlqr(As,Bs,Qs,Rs)
	
% PREDITTORE DI KALMAN
% (schema e grafici nei file-modello kalman_lin.mdl kalman_nonlin.mdl cfr_kalman.mdl(non fatto!!))
	
	input('PREDITTORE DI KALMAN -  matrice Lk - Soluz. dell''eq. di riccati Pk - autovalori dell''anello chiuso: (PREMI UN TASTO PER CONTINUARE)');
		
	Bs=[Bd; zeros(2,3)];
	Ds=Dd;
	Qk=eye(2,2);
	[Lk,Pk,Fk]=dlqr(Ad',Cd',Qk,Rs)

% Il modello sul sis. non lineare ha tempi di simulazione lunghissimi--> non si sa se funziona!!!!!
	
close all;

% end of file: Data.m
